//Class to provide the core functionality of the BlueJ Perforce extention.
package P4Bluej;
import bluej.extensions.*;
import bluej.extensions.event.*;
import bluej.extensions.editor.*;

import java.util.UUID;

import java.net.URL;
import javax.swing.*;
import java.awt.event.*;

import java.net.InetAddress;
import java.net.URISyntaxException;
import java.net.UnknownHostException;

import com.perforce.p4java.core.IMapEntry.EntryType;
import com.perforce.p4java.server.IServerInfo;

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.admin.IProtectionEntry;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.client.IClientSummary;
import com.perforce.p4java.client.IClientViewMapping;
import com.perforce.p4java.impl.mapbased.server.Server;
import com.perforce.p4java.core.ChangelistStatus;
import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.IJob;
import com.perforce.p4java.core.IUser;
import com.perforce.p4java.core.IUserGroup;
import com.perforce.p4java.core.IUserSummary;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.impl.generic.admin.ProtectionEntry;
import com.perforce.p4java.impl.generic.client.ClientView;
import com.perforce.p4java.impl.generic.client.ClientView.ClientViewMapping;
import com.perforce.p4java.impl.generic.core.Changelist;
import com.perforce.p4java.impl.generic.core.User;
import com.perforce.p4java.impl.generic.core.UserGroup;
import com.perforce.p4java.impl.mapbased.client.Client;
import com.perforce.p4java.option.client.SyncOptions;
import com.perforce.p4java.option.server.GetUserGroupsOptions;
import com.perforce.p4java.option.server.TrustOptions;
import com.perforce.p4java.option.server.UpdateUserOptions;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.ServerFactory;


	/**
		* Class to handle connection to, login and out and disconnection from a Perforce Server
		*/
		public class Connect
{
	//Instance variables to store data used for values that are required to connect and log in
	private String port;
	private String user;
	private String passwd;
private String client = System.getProperty("P4Bluej.clientName", "p4");
	//Not making these static for now, as a user may want to connect to multiple servers.

	//Constructor
    public Connect(final String aPort, final String aUser, final String aPasswd, final String aClient) 
	{
        this.port = aPort;
        this.user = aUser;
        this.passwd = aPasswd;
this.client = aClient;
        //Call the methods to connect and login.
        this.connect();
        //this.login();
    }
 
	/*Method to set the port*/
	public void setPort(String aPort)
		{
			this.port = aPort;
		}
		public String getPort()
			{
			return this.port;
			}
			//set the username
			public void setUserName(String aUser)
				{
					this.user = aUser;
				}
				//get username
				public String getUserName()
					{
						return this.user;
					}
//Getter and setter for the client
public void setClient(String aClient)
{
this.client = aClient;
}
public String getClient()
{
return this.client;
}

    /**
     * Connect to the server.
     */
    public void connect()
	{
	        try
	        {       
	//Create a Server object
	IOptionsServer server = ServerFactory.getOptionsServer(this.getPort(), null);
	            server.setUserName(this.getUserName());
	            server.connect();
	//Get p4 info
	                    IServerInfo info = server.getServerInfo();
	System.out.println(info);

		//Check if a client exists, if not we create one.
		//Not great long term, probably better to ask the user what they want, in case they want to switch to a different project etc.
		//Create a client object
			IClient client = server.getClient(this.getClient());
		 
           if (client != null)
		            {
		                server.setCurrentClient(client);
					}
		//if not, we need to create one.
		else
{
    IClient tempClient = new Client();
    // Setting up the name and the root folder
    tempClient.setName("tempClient" + UUID.randomUUID().toString().replace("-", ""));
    tempClient.setRoot("c:/temp");
    tempClient.setServer(server);
    // Setting the client as the current one for the server
    server.setCurrentClient(tempClient);
    // Creating Client View entry
    ClientViewMapping tempMappingEntry = new ClientViewMapping();
    // Setting up the mapping properties
    tempMappingEntry.setLeft("//depot/...");
    tempMappingEntry.setRight("//" + tempClient.getName() + "/...");
    tempMappingEntry.setType(EntryType.INCLUDE);
    // Creating Client view
    ClientView tempClientView = new ClientView();
    // Attaching client view entry to client view
    tempClientView.addEntry(tempMappingEntry);
    tempClient.setClientView(tempClientView);
    // Registering the new client on the server
    System.out.println(server.createClient(tempClient));
}

	//End try
}
    catch (RequestException rexc) {
        System.err.println(rexc.getDisplayString());
        rexc.printStackTrace();
    } catch (P4JavaException jexc) {
        System.err.println(jexc.getLocalizedMessage());
        jexc.printStackTrace();
    } catch (Exception exc) {
        System.err.println(exc.getLocalizedMessage());
        exc.printStackTrace();
    }
            }

			//End class
				}
